# -*- coding: utf-8 -*-
# Authors : Alix Chagué <alix.chague@inria.fr>
# Licence : MIT
"""
    The ``Parser Text`` module parse Text
    =====================================

"""

import os
from kami.kamutils._utils import (_report_log)

__all__ = [
    "_TextParser"
]

class _TextParser:
    """A very simple Text Parser for KaMI.

    Parameters
    ----------
        :param source:  path to source file or plain text
        :type source: str

    Attributes
    ----------
        :ivar file_name: source text file.
        :param file_name: str
        :ivar text: text content from source file.
        :param text: str
    """
    def _get_text(self) -> None:
        """Open a TXT file and load its content"""
        if self.file_name:
            with open(self.file_name, "r", encoding="utf8") as fh:
                content = fh.read()
                content = ''.join([line + "\n" for line in content.split('\n') if line.strip() != ''])
        self.text = content

    def __init__(self, source):
        self.file_name = None
        self.text = None
        if isinstance(source, str) and os.path.isfile(source):
            self.file_name = source
            self._get_text()
        elif isinstance(source, str):
            if os.sep in source:
                pass
                #_report_log("Provided input is considered as plain text. If you intended it to be a handled "+
                #            "as a path, you may need to make sure it is correct.", "W")
            self.text = source
        else:
            _report_log("TextParser can't proceed. Verify your input: it must be a string. "+
                        "Created an empty object.", "W")



# -*- coding: utf-8 -*-
# Authors : Alix Chagué <alix.chague@inria.fr>
#           Lucas Terriel <lucas.terriel@inria.fr>
# Licence : MIT
"""
    The ``Text recognizer`` module based on Kraken OCR/HTR engine
    =============================================================

    - Doc Kraken : http://kraken.re/master/api.html#api-quickstart 
    - Work only for ALTO and PAGE XML segmentation
    - Future : make prediction with text (no script for re-order lines
      in output for now) and default segmentation model (blla)

"""

from multiprocessing import Pool
from concurrent.futures import ThreadPoolExecutor, as_completed

import numpy as np
np.seterr(divide='ignore', invalid='ignore')
from PIL import Image

from kraken.lib import models
from kraken import rpred

from kami.kamutils._utils import (_report_log)


class _KrakenPrediction:
    """A OCR/HTR engine based on Kraken to create predictions from text recognizer model.


    Attributes
    ----------
        :ivar im: Image open as PIL object.
        :type im: PIL image object
        :ivar model: Load valid ocropus model and instanciate from the RNN configuration.
        :type model: A kraken.lib.models.TorchSeqRecognizer object
        :ivar bounds: Boundaries extract from ALTO or PAGE file.
        :type bounds: list
        :ivar pred_sentences: List of sentences predicted.
        :type pred_sentences: list
        :ivar pred_content: All prediction ; one sentence per line.
        :type pred_content: str
        :ivar verbosity: Details during prediction process. Defaults to False.
        :type verbosity: bool
        :ivar workers: Number of cpu workers use for inference. Defaults to 3.
        :type workers: int
    """
    def __init__(self, 
                    image_path : str, 
                    model_path : str, 
                    seg_bounds : list, 
                    verbosity: bool = False, 
                    workers: int = 7) -> None:
        self.im = Image.open(image_path)
        self.model = models.load_any(model_path)
        self.bounds = seg_bounds
        self.pred_sentences = []
        try:
            if verbosity:
                _report_log("Start with Kraken prediction...", type_log="I")

            # Create a pool process executor for turbo-charge (heavy cpu task)
            # Kraken transcription model inference function (repred.rpred)
            with Pool(processes=workers) as p:
                self.pred_sentences = p.map(self._transcribe, self.bounds)
                # if pool remove : self.pred_sentences = [self._transcribe(bound) for bound in self.bounds]

            if verbosity:
                _report_log("Kraken prediction finished with success.", type_log="I")
        except Exception as e:
            _report_log(f"[ERROR] Prediction with Kraken failed : {e}")
        
        # Retrieve all text with one sentence per line
        self.pred_content = "\n".join(self.pred_sentences)

    def _transcribe(self, bound):
        """Kraken method for recognition.

        Args:
            bound ([dict]): segment extract from ALTO/PAGE XML

        Returns:
            [str]: prediction.
        """
        return next(rpred.rpred(
            network=self.model, 
            im=self.im, 
            bounds=bound, 
            pad=16,
            bidi_reordering=True)).prediction



# -*- coding: utf-8 -*-
# Authors : Alix Chagué <alix.chague@inria.fr>
#           Lucas Terriel <lucas.terriel@inria.fr>
# Licence : MIT
"""Common code for Kami functions or classes.
"""

import time
from typing import Callable

from functools import wraps
import termcolor


__all__ = [
    "_report_log",
    "_timing"
]


def _report_log(message: str, type_log: str = "I") -> None:
    """Returns report log

    letter code to specify type of report

    'I' > Info | 'W' > Warning | 'E' > Error | 'S' > Success | 'V' > Verbose

    :Example:

    >>> _report_log("It seems your sentence is empty", "E")
    [ERROR   ⤬]  It seems your sentence is empty

    :param message: message to display in the log
    :type message: str
    :param type_log: type of message. Defaults to "I" (Info).
    :type type_log: str
    :return: None (print a log in defined color)
    :rtype: None
    """
    if type_log == "I":  # Info
        return print(f"[INFO    ℹ] {message}")
    if type_log == "W":  # Warning
        return termcolor.cprint(f"[WARNING ▲] {message}", "yellow")
    if type_log == "E":  # Error
        return termcolor.cprint(f"[ERROR   ⤬]  {message}", "red")
    if type_log == "S":  # Success
        return termcolor.cprint(f"[SUCCESS ✓]  {message}", "green")
    if type_log == "V":  # Verbose
        return termcolor.cprint(f"[DETAILS ℹ]  {message}", "blue")
    else:
        return print(message)


def _timing(function: Callable) -> Callable:
    """A simple decorator to compute execution time of any function
    """
    @wraps(function)
    def wrapper(*args, **kwargs):
        start = time.time()
        result = function(*args, **kwargs)
        end = time.time()
        _report_log(f'Total execution : {end-start} secs')
        return result
    return wrapper



# -*- coding: utf-8 -*-
# Authors : Lucas Terriel <lucas.terriel@inria.fr>
# Licence : MIT
"""
    The ``transformation`` module to preprocess text data
    =====================================================

"""


import re
import string
from typing import (Union,
                    List,
                    Mapping,
                    TypeVar,
                    Optional)

import unidecode

from kami.kamutils._utils import _timing

__all__ = [
    "ToCompose",
    "_AbstractTransform",
    "_Composer",
    "_SentencesToTokens",
    "RemovePunctuation",
    "RemoveDigits",
    "RemoveDiacritics",
    "RemoveNonUsefulWords",
    "RemoveSpecificWords",
    "Strip",
    "SubRegex",
    "ToLowerCase",
    "ToUpperCase"
]

# Declare preprocessing class types can only use with ToCompose() class
TRemovePunctuation = TypeVar('TRemovePunctuation', bound='RemovePunctuation')
TRemoveDiacritics = TypeVar('TRemoveDiacritics', bound='RemoveDiacritics')
TRemoveDigits = TypeVar('TRemoveDigits', bound='RemoveDigits')
TRemoveNonUsefulWords = TypeVar('TRemoveNonUsefulWords', bound='RemoveNonUsefulWords')
TRemoveSpecificWords = TypeVar('TRemoveSpecificWords', bound='RemoveSpecificWords')
TStrip = TypeVar('TStrip', bound='Strip')
TSubRegex = TypeVar('TSubRegex', bound='SubRegex')
TToLowerCase = TypeVar('TToLowerCase', bound='ToLowerCase')
TToUpperCase = TypeVar('TToUpperCase', bound='ToUpperCase')


class ToCompose:
    """

    """
    def __init__(self, sentences: list,
                 type_transforms: Optional[List[Union[
                     TRemovePunctuation,
                     TRemoveDigits,
                     TRemoveNonUsefulWords,
                     TRemoveDiacritics,
                     TRemoveSpecificWords,
                     TStrip,
                     TSubRegex,
                     TToLowerCase,
                     TToUpperCase]]]):
        process = _Composer(type_transforms)(sentences)
        self.reference = process[0]
        self.prediction = process[1]


class _AbstractTransform(object):
    def __call__(self, sentences: Union[str, List[str]]):
        if isinstance(sentences, str):
            return self.process_string(sentences)
        elif isinstance(sentences, list):
            return self.process_list(sentences)
        else:
            raise ValueError(
                f"input {sentences} was expected to be a string or list of strings"
            )

    def process_string(self, sequence: str):
        raise NotImplementedError()

    def process_list(self, group: List[str]):
        return [self.process_string(sequence) for sequence in group]


class _Composer(object):
    def __init__(self, transforms: List[_AbstractTransform]):
        self.transforms = transforms

    def __call__(self, text):
        for transform in self.transforms:
                text = transform(text)
        return text


#
# Tokenizers
#


class _SentencesToTokens(_AbstractTransform):
    """

    """
    def __init__(self, delimiter: str = " "):
        self.delimiter = delimiter

    def process_string(self, sequence: str):
        return sequence.split(self.delimiter)


#
# Transformations
#


class RemovePunctuation(_AbstractTransform):
    """Remove punctuation from text. Default remove : !"#$%&'()*+, -./:;<=>?@[\]^_`{|}~
    User can specify punctuation want to keep in text.

    User can directly access to this class or via :class: `ToCompose` class.

    Parameters
    ----------
    :param keep_punctuation: list of punctuation to keep in text.
    :type keep_punctuation: list, optional

    Attributes
    ----------
    See Parameters
    """
    def __init__(self, keep_punctuation: Optional[List[str]] = ''):
        self.keep_punctuation = keep_punctuation

    def process_string(self, sequence: str):
        default_punctuation = string.punctuation
        if len(self.keep_punctuation) == 0:
            table = str.maketrans('', '', default_punctuation)
        else:
            new_punctuation = "".join(
                [symbol for symbol in default_punctuation if symbol not in self.keep_punctuation])
            table = str.maketrans('', '', new_punctuation)

        sequence = sequence.translate(table)

        return sequence


class RemoveDiacritics(_AbstractTransform):
    """Remove digits from text.

    User can directly access to this class or via :class: `ToCompose` class.
    """
    def process_string(self, sequence: str):

        sequence = unidecode.unidecode(sequence)
        sequence = re.sub(r'<{2,}', "", sequence)
        sequence = re.sub(r'>{2,}', "", sequence)
        sequence = re.sub(r'-{2,}', "", sequence)
        #sequence = unicodedata.normalize('NFD', sequence)\
           #.encode('ascii', 'ignore')\
           #.decode("utf-8")
        return str(sequence)


class RemoveDigits(_AbstractTransform):
    """Remove digits from text.

    User can directly access to this class or via :class: `ToCompose` class.
    """
    def process_string(self, sequence: str):
        sequence = re.sub(r"\d+", "", sequence)
        return sequence


class RemoveNonUsefulWords(_AbstractTransform):
    """Remove empties tokens from text.

    User can directly access to this class or via :class: `ToCompose` class.
    """
    def process_string(self, sequence: str):
        sequence = " ".join([token for token in sequence.split() if token != ''])
        return sequence


class RemoveSpecificWords(_AbstractTransform):
    """Remove specifics words that predefined by user.

    User can directly access to this class or via :class: `ToCompose` class.

    Parameters
    ----------
    :param words_to_remove: list of words to remove.
    :type words_to_remove: list

    Attributes
    ----------
    See Parameters

    """
    def __init__(self, words_to_remove: List[str]):
        self.words_to_remove = words_to_remove

    def process_string(self, sequence: str):
        # TODO(@Luca) : look better tokenization eg. "Curée," to ["Curée", ","]
        sequence = " ".join([token for token in sequence.split() if token not in self.words_to_remove])
        return sequence


class Strip(_AbstractTransform):
    """Performs text strip.

    User can directly access to this class or via :class: `ToCompose` class.
    """
    def process_string(self, sequence: str):
        return sequence.strip()


class SubRegex(_AbstractTransform):
    """Performs substitutions in text with regex patterns.

    User can directly access to this class or via :class: `ToCompose` class.

    Parameters
    ----------
    :param substitutions: a dictionary where the key is a regex pattern
    and value is the element to substitute.
    :type substitutions: dict

    Attributes
    ----------
    See Parameters
    """
    def __init__(self, substitutions: dict):
        self.substitutions = substitutions

    def process_string(self, sequence: str):
        sequence = " ".join([re.sub(key, value, sequence) for key, value in self.substitutions.items()])
        return sequence


class ToLowerCase(_AbstractTransform):
    """Pass a string to lowercase.

    User can directly access to this class or via :class: `ToCompose` class.
    """
    def process_string(self, sequence: str):
        return sequence.lower()


class ToUpperCase(_AbstractTransform):
    """Pass a string to uppercase.

    User can directly access to this class or via :class: `ToCompose` class.
    """
    def process_string(self, sequence: str):
        return sequence.upper()


# Utils functions relative to transformation

def count_diacritics(string):
    """A simple diacritics counter"""
    total_diacritics = []
    for char in string:
        char_transform = unidecode.unidecode(char)
        #char_transform = unicodedata.normalize('NFD', char)\
           #.encode('ascii', 'ignore')\
           #.decode("utf-8")
        if char != char_transform:
            total_diacritics.append(char)

    return len(total_diacritics)



import unittest

from kami.preprocessing.transformation import (ToLowerCase,
                                               ToUpperCase,
                                               RemovePunctuation,
                                               RemoveDiacritics,
                                               RemoveDigits,
                                               RemoveNonUsefulWords,
                                               RemoveSpecificWords,
                                               Strip,
                                               SubRegex,
                                               ToCompose)


class testPreprocessing(unittest.TestCase):
    def setUp(self) -> None:
        self.sentence = "Les 13 ans de Maxime ? étaient, Déjà terriblement, savants ! - La Curée, 1871. En avant, pour la lecture."

    def test_removeLowercase(self):
        transform = ToLowerCase()(self.sentence)
        self.assertEqual(transform, "les 13 ans de maxime ? étaient, déjà terriblement, savants ! - la curée, 1871. en avant, pour la lecture.")

    def test_removeUppercase(self):
        transform = ToUpperCase()(self.sentence)
        self.assertEqual(transform, "LES 13 ANS DE MAXIME ? ÉTAIENT, DÉJÀ TERRIBLEMENT, SAVANTS ! - LA CURÉE, 1871. EN AVANT, POUR LA LECTURE.")

    def test_removePunct(self):
        transform = RemovePunctuation()(self.sentence)
        self.assertEqual(transform, "Les 13 ans de Maxime  étaient Déjà terriblement savants   La Curée 1871 En avant pour la lecture")

    def test_removeDiacritics(self):
        transform = RemoveDiacritics()(self.sentence)
        self.assertEqual(transform, "Les 13 ans de Maxime ? etaient, Deja terriblement, savants ! - La Curee, 1871. En avant, pour la lecture.")

    def test_removeDigits(self):
        transform = RemoveDigits()(self.sentence)
        self.assertEqual(transform, "Les  ans de Maxime ? étaient, Déjà terriblement, savants ! - La Curée, . En avant, pour la lecture.")

    def test_removeNonUsefulWords(self):
        transform = RemoveNonUsefulWords()(self.sentence)
        self.assertEqual(transform, "Les 13 ans de Maxime ? étaient, Déjà terriblement, savants ! - La Curée, 1871. En avant, pour la lecture.")

    def test_removeSpecificWords(self):
        words_to_remove = ["Maxime", "Déjà"]
        transform = RemoveSpecificWords(words_to_remove=words_to_remove)(self.sentence)
        self.assertEqual(transform, "Les 13 ans de ? étaient, terriblement, savants ! - La Curée, 1871. En avant, pour la lecture.")

    def test_strip(self):
        transform = Strip()(self.sentence)
        self.assertEqual(transform, "Les 13 ans de Maxime ? étaient, Déjà terriblement, savants ! - La Curée, 1871. En avant, pour la lecture.")

    def test_subRegex(self):
        substitutions = {"[L|l]\w+": "-- REPLACE --"}
        transform = SubRegex(substitutions=substitutions)(self.sentence)
        self.assertEqual(transform, "-- REPLACE -- 13 ans de Maxime ? étaient, Déjà terrib-- REPLACE --, savants ! - -- REPLACE -- Curée, 1871. En avant, pour -- REPLACE -- -- REPLACE --.")

    def test_multipleReplace(self):
        transform = ToCompose([self.sentence, ""], [RemoveDiacritics(), RemovePunctuation(), ToLowerCase(), RemoveNonUsefulWords()])
        self.assertEqual(transform.reference, "les 13 ans de maxime etaient deja terriblement savants la curee 1871 en avant pour la lecture")








# -*- coding: utf-8 -*-
# Authors : Lucas Terriel <lucas.terriel@inria.fr>
# Contibutors : Thibault Clérice
# Licence : MIT
"""Common code for all metrics.
"""

import decimal
from typing import Sequence, List

__all__ = [
    "_WordRegister",
    "_hot_encode",
    "_truncate_score",
    "_get_percent"
]


class _WordRegister:
    """A simple dictionnary with auto-incremental index"""
    def __init__(self):
        self._register = {}

    def __getitem__(self, key: str):
        if key in self._register:
            val = self._register[key]
        else:
            self._register[key] = val = chr(0x0000 + len(self._register))
        return val

    def __str__(self):
        return f'Actual register : {self._register}'


def _hot_encode(word_lists: Sequence[Sequence[str]]) -> List[str]:
    """Pre-process the truth and hypothesis into a words form that Levenshtein can handle.

    Take word_lists, transform them into hot-encoded strings.

    :Example:

    >>> list(_hot_encode([["w1", "w2", "w3"], ["w4", "w5", "w1"]]))
    ['\x00\x01\x02', '\x03\x04\x00']

    :param word_lists: List of List of words (generally 2)
    :type word_lists: list
    :return: hot-encoded string
    :rtype: list
    """
    wtox = _WordRegister()
    for word_list in word_lists:
        yield "".join([wtox[word] for word in word_list])


def _truncate_score(score: float, round_digits: str) -> float:
    """truncate the result with predifined digits after the decimal point (does not display zeros)

    :Exemple:

    >>> _truncate_score(3.1415926, round_digits='.001')
    3.141
    >>> _truncate_score(3.1415926, round_digits='.01')
    3.14

    :param score: result of computation to truncate
    :type score: float
    :param round_digits: number of digits after the decimal point (exemple format : ".001")
    :type round_digits: str
    :return: result truncated
    :rtype: float
    """
    result_truncate = float(
        decimal.Decimal(score).quantize(
            decimal.Decimal(round_digits),
            rounding=decimal.ROUND_DOWN))
    return result_truncate


def _get_percent(score: float) -> float:
    """Return score in percentage format

    :Example:

    >>> _get_percent(0.234)
    23.4

    :param score: result of computation
    :type: float
    :return: result in percentage
    :rtype: float
    """
    return score * 100



import unittest

from kami.Kami import Kami


class testKamiClient(unittest.TestCase):
    def setUp(self) -> None:
        self.maxDiff = None

        self.reference = "Six semaines plus tard, Claude peignait un matin dans un flot de soleil qui tombait par la baie vitrée de l’atelier."
        self.prediction = "Six semaiNEs plus tard, lCCaude peignait un MA dans un flotille de soleil qui tombait baie vitrée de l’atelier."

        self.gt_page = "../datatest/lectaurep_set/image_gt_page1/FRAN_0187_16402_L-0_page.xml"
        self.image_page = "../datatest/lectaurep_set/image_gt_page1/FRAN_0187_16402_L-0.png"
        self.model_page = "../datatest/lectaurep_set/models/mixte_mrs_15.mlmodel"

    """
    def test_sentences(self):
        k1 = Kami([self.reference, self.prediction], verbosity=False, truncate=True, percent=True, round_digits='0.01')
        self.assertEqual(k1.scores.board, {'levensthein_distance_char': 20,
                                          'levensthein_distance_words': 6,
                                          'hamming_distance': 'Ø',
                                          'wer': 28.57,
                                          'cer': 17.24,
                                          'wacc': 71.42,
                                          'mer': 16.52,
                                          'cil': 20.77,
                                          'cip': 79.22,
                                          'hits': 101,
                                          'substitutions': 5,
                                          'deletions': 10,
                                          'insertions': 5})

    def test_page(self):
        k2 = Kami(self.gt_page, model=self.model_page, image=self.image_page, verbosity=False, truncate=True, percent=True, round_digits='0.01')

        self.assertEqual(k2.scores.board, {'cer': 0.49,
                                           'cil': 0.59,
                                           'cip': 99.4,
                                           'deletions': 0,
                                           'hamming_distance': 'Ø',
                                           'hits': 1002,
                                           'insertions': 4,
                                           'levensthein_distance_char': 5,
                                           'levensthein_distance_words': 2,
                                           'mer': 0.49,
                                           'substitutions': 1,
                                           'wacc': 98.94,
                                           'wer': 1.05})
    """


# -*- coding: utf-8 -*-
# Authors : Lucas Terriel <lucas.terriel@inria.fr>
# Licence : MIT
"""
    The ``evaluation`` module to assess text recognition (OCR/HTR) scores
    ======================================================================

"""

from typing import Sequence, Tuple, Union
from Levenshtein import (distance, 
                         hamming, 
                         editops)
from ._base_metrics import (_truncate_score,
                            _hot_encode,
                            _get_percent)

__all__ = [
    "Scorer",
]


class Scorer:
    """This class calculates the set of HTR / OCR scores.

    User can accesses directly to metrics/scores via the attributes of the :class: `Scorer` class
    in constructor method or via :class: `Kami` facade class.

    Parameters
    ----------

        :param reference: Ground-truth text or string.
        :type reference: str
        :param prediction: model string or text prediction or test text or string.
        to compare to :param: `reference` parameter
        :type prediction: str
        :param insertion_cost: predefined a weight for insertions errors. Defaults to 1.0.
        :type insertion_cost: float
        :param substitution_cost: predefined a weight for substitution errors. Defaults to 1.0.
        :type substitution_cost: float
        :param deletion_cost: predefined a weight for insertions errors. Defaults to 1.0.
        :type deletion_cost: float
        :param show_percent: `True` if the user want to show result in percent else `False`, defaults to False.
        :type show_percent: bool, optional
        :param truncate_score: `True` if the user want to truncate result in percent else `False`,
        can indicate the number of digits after floating point with :param: `round_digits` parameter, defaults to False.
        :type truncate_score: bool, optional
        :param round_digits: Set the number of digits after floating point in string form, defaults to '.01'.
        :type round_digits: str, optional

    Attributes
    ----------

        :ivar _opt_percent: Option to show result in percent. 
        :type _opt_percent: bool
        :ivar _opt_truncate: Option to truncate result. 
        :type _opt_truncate: bool
        :ivar _round_digits: Number of digits after floating point. 
        :type _round_digits: str
        :ivar reference: Ground-truth text or string.
        :type reference: str
        :ivar prediction: Text to compare or predicted sequence.
        :type prediction: str
        :ivar insertion_cost: predefined a weight for insertions errors. 
        :type insertion_cost: float
        :ivar substitution_cost: predefined a weight for substitution errors. 
        :type substitution_cost: float
        :ivar deletion_cost: predefined a weight for insertions errors. 
        :type deletion_cost: float
        :ivar length_char_reference: Total number of characters in reference string
        :type length_char_reference: int
        :ivar length_char_prediction: Total number of characters in predicted string
        :type length_char_prediction: int
        :ivar length_words_reference: Total number of words in reference string
        :type length_words_reference: int
        :ivar length_words_prediction: Total number of words in predicted string
        :type length_words_prediction: int
        :ivar lev_distance_words: Edit distance (levensthein) between reference and prediction based on words
        :type lev_distance_words: int
        :ivar lev_distance_char: Edit distance (levensthein) between reference and prediction based on characters
        :type lev_distance_char: int
        :ivar hamming: Hamming distance between reference and prediction based on words
        :type hamming: int or str ("Ø") if sequences have not the same numbers of characters
        :ivar wer: Word Error Rate
        :type wer: float
        :ivar wer_hunt: Hunt word error rate
        :ivar cer: Character Error Rate
        :type cer: float
        :ivar wacc: Word Accuracy
        :type wacc: float
        :ivar cip: Character Information Preserved
        :type cip: float
        :ivar cil: Character Information Lost
        :type cil: float
        :ivar mer: Match Error Rate
        :type mer: float
        :ivar hits: number of characters that matches between reference and prediction string
        :type hits: int
        :ivar substs: number of characters substituted between reference and prediction string
        :type substs: int
        :ivar deletions: number of characters deleted between reference and prediction string
        :type deletions: int
        :ivar insertions: number of characters inserted between reference and prediction string
        :type insertions: int
        :ivar board: A benchmark of all metrics
        :type board: dict


    """
    def __init__(self,
                 reference: str,
                 prediction: str,
                 insertion_cost: int = 1.0,
                 deletion_cost: int = 1.0,
                 substitution_cost: int = 1.0,
                 show_percent: bool = False,
                 truncate_score: bool = False,
                 round_digits: str = '.01') -> None:

        # Scores display options
        self._opt_percent  = show_percent
        self._opt_truncate = truncate_score
        self._round_digits = round_digits
        
        # Strings to compare
        self.reference  = reference
        self.prediction = prediction

        # Operations weigthts pre-defined (default to 1)
        self.insertion_cost    = insertion_cost
        self.deletion_cost     = deletion_cost
        self.substitution_cost = substitution_cost

        # Length set of sentences
        self.length_char_reference   = len(reference)
        self.length_char_prediction  = len(prediction)
        self.length_words_reference  = len(reference.split())
        self.length_words_prediction = len(prediction.split())

        # Strings operations (weighted and unweighted / char-based and word-based)
        self.hits, self.substs, self.deletions, self.insertions, self.substs_weighted, self.deletions_weighted, self.insertions_weighted, self.word_substs, self.word_deletions, self.word_insertions, self.word_substs_weighted, self.word_deletions_weighted, self.word_insertions_weighted = self._get_operation_counts()

        # Distances
        if self.insertion_cost == 1 and self.deletion_cost  == 1 and self.substitution_cost == 1:
            self.lev_distance_words, self.lev_distance_char = self._levensthein_distance()
        else:
            self.lev_distance_words, self.lev_distance_char = self._weighted_levensthein_distance()

        self.hamming = self._hamming_distance()

        # HTR/OCR Metrics
        self.wer      = self._wer()
        self.wer_hunt = self._wer_hunt()
        self.cer      = self._cer()
        self.wacc     = self._wacc()
        self.cip      = self._cip()
        self.cil      = self._cil()
        self.mer      = self._mer()

        if self._opt_percent:
            self.wer       = _get_percent(self.wer)
            self.wer_hunt  = _get_percent(self.wer_hunt)
            self.cer       = _get_percent(self.cer)
            self.wacc      = _get_percent(self.wacc)
            self.cip       = _get_percent(self.cip)
            self.cil       = _get_percent(self.cil)
            self.mer       = _get_percent(self.mer)

        if self._opt_truncate:
            self.wer       = _truncate_score(self.wer, self._round_digits)
            self.wer_hunt  = _truncate_score(self.wer_hunt, self._round_digits)
            self.cer       = _truncate_score(self.cer, self._round_digits)
            self.wacc      = _truncate_score(self.wacc, self._round_digits)
            self.cip       = _truncate_score(self.cip, self._round_digits)
            self.cil       = _truncate_score(self.cil, self._round_digits)
            self.mer       = _truncate_score(self.mer, self._round_digits)

        # Summary of all metrics
        self.board = {
                "levensthein_distance_char": self.lev_distance_char,
                "levensthein_distance_words": self.lev_distance_words,
                "hamming_distance": self.hamming,
                "wer": self.wer,
                "cer": self.cer,
                "wacc": self.wacc,
                "wer_hunt": self.wer_hunt,
                "mer": self.mer,
                "cil": self.cil,
                "cip": self.cip,
                "hits": self.hits,
                "substitutions": self.substs,
                "deletions": self.deletions,
                "insertions": self.insertions,
                "Length_reference": self.length_char_reference,
                "Length_prediction": self.length_char_prediction
        }

    # Collection of distance metrics # 
    def _levensthein_distance(self) -> Tuple[float, float]:
        """Compute Levensthein distance from C extension module Python-Levensthein.
        
        Returns:
            Tuple[float, float]: weighted levensthein distance based on char level, weighted levensthein distance based on word level
        """
        return distance(*_hot_encode(
            [self.reference.split(), self.prediction.split()])), distance(self.reference, self.prediction)

    def _weighted_levensthein_distance(self) -> Tuple[float, float]:
        """Compute Levensthein distance from predefined cost.

        Returns:
            Tuple[float, float]: weighted levensthein distance based on word level, weighted levensthein distance based on char level
        """
        return sum(
            [self.word_substs_weighted,
            self.word_deletions_weighted,
            self.word_insertions_weighted]
            ), sum(
            [self.substs_weighted,
            self.deletions_weighted,
            self.insertions_weighted]
            )

    def _hamming_distance(self) -> Union[str, int]:
        """Compute Hamming distance from C extension module Python-Levensthein."""
        return "Ø" if self.length_char_reference != self.length_char_prediction else hamming(self.reference, self.prediction)

    # Collection of HTR/OCR metrics #

    def _wer(self) -> float:
        """Compute word error rate (WER)."""
        return (self.lev_distance_words/self.length_words_reference)

    def _wer_hunt(self) -> float:
        """Compute Hunt word error rate that minimize errors of deletions and insertions."""
        return (sum([
                self.word_substs,
                0.5 * self.word_deletions, 
                0.5 * self.word_insertions
                ]
                )/self.length_words_reference)if (
                    self.insertion_cost == 1 
                    and self.deletion_cost  == 1 
                    and self.substitution_cost == 1)else (sum(
                        [
                            self.word_substs_weighted,
                            0.5 * self.word_deletions_weighted,
                            0.5 * self.word_insertions_weighted
                        ]
                        )/self.length_words_reference)

    def _cer(self) -> float:
        """Compute character error rate (CER)."""
        return (self.lev_distance_char/self.length_char_reference)

    def _wacc(self) -> float:
        """Compute word accuracy (Wacc)."""
        return (1 - (self.lev_distance_words/self.length_words_reference))

    # Collection of experimental ASR (Automatic Speech Recognition) metrics #
    def _cip(self) -> float:
        """Compute character information preserved (CIP)."""
        return (float(self.hits)/self.length_char_reference)*(float(self.hits)/self.length_char_prediction) if self.prediction else 0.0

    def _cil(self) -> float:
        """Compute character information lost (CIL)."""
        return (1 - (float(self.hits)/self.length_char_reference)*(float(self.hits)/self.length_char_prediction)) if self.prediction else 0.0

    def _mer(self) -> float:
        """Compute match error rate (MER)."""
        return float(
            self.substs
            + self.deletions
            + self.insertions) \
            / float(
            self.hits
            + self.substs
            + self.deletions
            + self.insertions)

    @staticmethod
    def sentence_blocks(sentence, n=10):
        s = sentence.split("\n")
        for i in range(0, len(s), n):
            yield " ".join(s[i:i + n])

    def _get_operation_counts(self) -> Tuple[int, int, int, int, float, float, float, int, int, int, float, float, float]:
        """Find sequence of edit operations transforming one string to another.
        Based on editops function from C extension module python-Levenshtein."""

        # Texts over ~7000/8000 characters can cause a MemoryError with
        # editops function; the strategy is to tokenize sentences and pass
        # in editops with a batch process.
        try:
            result_editops_char = editops(self.reference, self.prediction)
            result_editops_word = editops(*_hot_encode(
                [
                    self.reference.split(),
                    self.prediction.split()
                ]
            )
                                          )
        except MemoryError:
            result_editops_char = []
            result_editops_word = []

            for r, p in zip(self.sentence_blocks(self.reference), self.sentence_blocks(self.prediction)):
                result_editops_char.extend(editops(r, p))
                result_editops_word.extend(editops(*_hot_encode(
                    [
                        r.split(),
                        p.split()
                    ]
                )
                                                   ))




        def _sum_operations(keyword: str, results: Sequence[Tuple[str, int, int]]) -> int:
            """Compute a sum of define operations"""
            total = sum(1 if operations[0] == keyword else 0 for operations in results)
            return total

        def _sum_operations_weighted(keyword: str, results: Sequence[Tuple[str, int, int]], weight: float) -> float:
            total = sum(1 * weight if operations[0] == keyword else 0 for operations in results)
            return total

        substitutions = _sum_operations("replace", result_editops_char)
        deletions     = _sum_operations("delete", result_editops_char)
        insertions    = _sum_operations("insert", result_editops_char)

        substitutions_weighted = _sum_operations_weighted("replace", result_editops_char, weight=self.substitution_cost)
        deletions_weighted     = _sum_operations_weighted("delete", result_editops_char, weight=self.deletion_cost)
        insertions_weighted    = _sum_operations_weighted("insert", result_editops_char, weight=self.insertion_cost)

        word_substitutions = _sum_operations("replace", result_editops_word)
        word_deletions     = _sum_operations("delete", result_editops_word)
        word_insertions    = _sum_operations("insert", result_editops_word)

        word_substitutions_weighted = _sum_operations_weighted("replace", result_editops_word, weight=self.substitution_cost)
        word_deletions_weighted     = _sum_operations_weighted("delete", result_editops_word, weight=self.deletion_cost)
        word_insertions_weighted    = _sum_operations_weighted("insert", result_editops_word, weight=self.insertion_cost)

        hits = self.length_char_reference - (substitutions + deletions)
        return hits, substitutions, deletions, insertions, substitutions_weighted, deletions_weighted, insertions_weighted, word_substitutions, word_deletions, word_insertions, word_substitutions_weighted, word_deletions_weighted, word_insertions_weighted



# -*- coding: utf-8 -*-
# Authors : Alix Chagué <alix.chague@inria.fr>
#           Lucas Terriel <lucas.terriel@inria.fr>
# Licence : MIT
"""Client interface calls several sub-system of Kami-lib
"""
from typing import Union

from kami.parser import (parser_text,
                         parser_xml)
from kami.preprocessing.transformation import (RemoveDigits,
                                               ToLowerCase,
                                               ToUpperCase,
                                               RemovePunctuation,
                                               RemoveDiacritics,
                                               _Composer,
                                               count_diacritics)
from kami.transcription.prediction import _KrakenPrediction
from kami.metrics.evaluation import Scorer

import warnings
warnings.filterwarnings("ignore")

class Kami:
    """A Facade class provides a simple interface to the complex logic of one or
    several subsystems in Kami.

    This hide a complexity of Kami subsystems as :
    * Preprocessing (optionnal)
    * Prediction (if Kraken is use)
    * Metrics

    Parameters
    ----------
        :param data: Data to evaluate as two strings or two text file in list eg. ["./gt.txt", "./pred.txt"]; or a path to single ALTO or PAGE XML.
        :type data: Union[str,list]
        :param image: Path to image use to prediction if Kraken is use.
        :type image: str
        :param model: Path to transcription model use by Kraken.
        :type model: str
        :param apply_transforms: Code to apply textual variations eg. "XPD" 
        (List transformations : D : remove digits / U : uppercase / L : lowercase / P : remove punctuation / X : remove diacritics). Defaults to "".
        :type: str
        :param workers: Number of cpu workers use for inference. Defaults to 3.
        :type workers: int
        :param text_direction: principal text direction for column ordering use by Kraken : 
        "horizontal-lr", "horizontal-rl", "vertical-lr", "vertical-rl". Defaults to "horizontal-lr".
        :type: str
        :param script: script use by Kraken. Defaults to "default". 
        :type script: str
        :param verbosity: Display logs message during execution. Defaults to False. 
        :type verbosity: bool
        :param insertion_cost: predefined a weight for insertions errors. Defaults to 1.0.
        :type insertion_cost: float
        :param substitution_cost: predefined a weight for substitutions errors. Defaults to 1.0.
        :type substitution_cost: float
        :param deletion_cost: predefined a weight for deletions errors. Defaults to 1.0. 
        :type deletion_cost: float
        :param truncate: Option to truncate result. Defaults to "False". 
        :type truncate: str
        :param percent: `True` if the user want to show result in percent else `False`. Defaults to False.
        :type percent: bool
        :param round_digits: Set the number of digits after floating point in string form. Defaults to to '.01'.
        :type round_digits: str

    Attributes
    ----------
        :ivar reference: ground truth text.
        :type reference: str
        :ivar prediction: prediction by htr/ocr model.
        :type prediction: str
        :ivar model: path to the transcription model.
        :type model: str
        :ivar apply_transforms: see also `Parameters` section for more details.
        :type apply_transforms: list
        :ivar workers: see also `Parameters` section for more details.
        :type workers: int
        :ivar text_direction: see also `Parameters` section for more details.
        :type text_direction: str
        :ivar script: see also `Parameters` section for more details.
        :type script: str
        :ivar verbosity: see also `Parameters` section for more details.
        :type verbosity: bool
        :ivar insertion_weigtht: see also `Parameters` section for more details.
        :type insertion_weigtht: float
        :ivar substitution_weigtht: see also `Parameters` section for more details.
        :type substitution_weigtht: float
        :ivar deletion_weight: see also `Parameters` section for more details.
        :type deletion_weight: float
        :ivar truncate: see also `Parameters` section for more details.
        :type truncate: bool
        :ivar percent: see also `Parameters` section for more details.
        :type percent: bool
        :ivar round_digits: see also `Parameters` section for more details.
        :type round_digits: str
        :ivar reference_preprocess: ground truth with text preprocessing applied
        :type reference_preprocess: str
        :ivar prediction_preprocess: prediction with text preprocessing applied
        :type prediction_preprocess: str
        :ivar scores: KaMI Scorer object that contains all metrics.
        :type scores: KaMI Scorer object 

    """

    def __init__(self,
                 data: Union[str, list],
                 image: str = "",
                 model: str = "",
                 apply_transforms: str = "",
                 workers: int = 3,
                 text_direction : str = "horizontal-lr",
                 script= "default",
                 verbosity: bool = False,
                 insertion_cost: float = 1.0,
                 substitution_cost: float = 1.0,
                 deletion_cost: float = 1.0,
                 truncate: bool = False,
                 percent: bool = False,
                 round_digits: str = '.01'
                 ) -> None:

        # Data inputs
        self.reference = None
        self.prediction = None
        self.model = None

        # Preprocessing options inputs
        self.apply_transforms = [code for line in apply_transforms.split() for code in line]

        # Kraken options inputs
        self.workers = workers
        self.text_direction = text_direction
        self.script = script

        # Debug options
        self.verbosity = verbosity

        # Options for score weighting
        self.insertion_weigtht=insertion_cost
        self.substitution_weigtht=substitution_cost
        self.deletion_weight=deletion_cost

        # Display options for scores output
        self.truncate = truncate
        self.percent = percent
        self.round_digits = round_digits

        # Output
        self.reference_preprocess = ""
        self.prediction_preprocess = ""
        self.scores = None

        if isinstance(data, list) and len(data) > 1:
            # case with two text files => compute score
            if data[0].endswith('txt') and data[1].endswith('txt'):
                self.reference = parser_text._TextParser(data[0]).text
                self.prediction = parser_text._TextParser(data[1]).text
            # case with two strings => compute score
            else:
                self.reference = data[0]
                self.prediction = data[1]

            self.scores = Scorer(self.reference,
                                 self.prediction,
                                 insertion_cost=self.insertion_weigtht,
                                 deletion_cost=self.deletion_weight,
                                 substitution_cost=self.substitution_weigtht,
                                 truncate_score=self.truncate,
                                 show_percent=self.percent,
                                 round_digits=self.round_digits)

        # case with GT XML PAGE / XML ALTO => create a HTR pipeline => compute scores
        elif isinstance(data, str) and data.endswith('xml'):
            self.reference_parse = parser_xml._XMLParser(xml_path=data, 
                                                         text_direction=self.text_direction, 
                                                         script=self.script)
            self.file_name = self.reference_parse.filename
            self.reference = self.reference_parse.content
            bounds = self.reference_parse.list_bounds
            pipeline = _KrakenPrediction(image_path=image,
                                         model_path=model,
                                         workers=self.workers,
                                         seg_bounds=bounds,
                                         verbosity=self.verbosity)
            self.prediction = pipeline.pred_content
            self.scores = Scorer(self.reference,
                                 self.prediction,
                                 insertion_cost=self.insertion_weigtht,
                                 deletion_cost=self.deletion_weight,
                                 substitution_cost=self.substitution_weigtht,
                                 truncate_score=self.truncate,
                                 show_percent=self.percent,
                                 round_digits=self.round_digits)


        else:
            raise ValueError("Something is wrong. Check your data (ground truth and/or prediction).")

        # Case with preprocessing and modulate .board dict of Scorer object
        if len(apply_transforms) > 0:
            # Create a new dict to save the different state of computations during the transformations
            new_score = dict()

            # Create a new list to save the functions of transformations
            to_compose = list()

            # A dictionary associate the user code / the preprocessing function / readable name of function
            # Codes legend for users :  D : digits / U : uppercase / L : lowercase / P : punctuation / X : diacritics
            CODES_TRANSFORMS = {
                "D": (RemoveDigits(), "non_digits"),
                "U": (ToUpperCase(), "uppercase"),
                "L": (ToLowerCase(), "lowercase"),
                "P": (RemovePunctuation(), "remove_punctuation"),
                "X": (RemoveDiacritics(), "remove_diacritics")
            }

            # Initialize the default dict that corresponding to computations on sequences before the applications of
            # transformations
            new_score["default"] = self.scores.board

            for code in CODES_TRANSFORMS.items():
                if code[0] in apply_transforms:
                    to_compose.append(code[1][0])
                    # Retrieve the scores of one transformation and add this in new dict with readable name
                    new_score[code[1][1]] = self._compute_state_transformations(code[1][0]).board
            # Retrieve the scores of all transformations in same time and add this in new dict with readable name
            scores_all_transforms, sequences_all_transforms = self._compute_all_transformations(to_compose)
            new_score["all_transforms"] = scores_all_transforms.board


            # Count total char transformed with
            # transformations in reference and prediction :
            # Add transformed sentences
            self.reference_preprocess = sequences_all_transforms[0]
            self.prediction_preprocess = sequences_all_transforms[1]


            if "D" or "P" in apply_transforms:
                # Compute the different lengths between string
                # (initial reference and transformed reference / initial reference and transformed prediction)
                # to compute the number of digits or punctuation remove
                total_char_removed_reference \
                    = (len(self.reference)
                        - len(self.reference_preprocess))
                total_char_removed_prediction \
                    = (len(self.prediction)
                        - len(self.prediction_preprocess))

                new_score["Total_char_removed_from_reference"] = total_char_removed_reference
                new_score["Total_char_removed_from_prediction"] = total_char_removed_prediction

            if "X" in apply_transforms:
                # Compute the number of diacritics removed
                total_diacritics_reference \
                    = (
                        count_diacritics(self.reference)
                        - count_diacritics(self.reference_preprocess))
                total_diacritics_prediction \
                    = (
                        count_diacritics(self.prediction)
                        - count_diacritics(self.prediction_preprocess))
                new_score["Total_diacritics_removed_from_reference"] = total_diacritics_reference
                new_score["Total_diacritics_removed_from_prediction"] = total_diacritics_prediction

            if "U" in apply_transforms:
                def count_lower(string):
                    total = sum(1 for c in string if c.islower())
                    return total
                # Compute total of lowercase char pass in uppercase in reference after transform
                total_lower_reference = count_lower(self.reference)
                total_lower_prediction = count_lower(self.prediction)
                new_score["Total_char_pass_in_uppercase_in_reference"] = total_lower_reference
                new_score["Total_char_pass_in_uppercase_in_prediction"] = total_lower_prediction

            if "L" in apply_transforms:
                def count_upper(string):
                    total = sum(1 for c in string if c.isupper())
                    return total
                # Compute total of lowercase char pass in uppercase in reference after transform
                total_upper_reference = count_upper(self.reference)
                total_upper_prediction = count_upper(self.prediction)
                new_score["Total_char_pass_in_lowercase_in_reference"] = total_upper_reference
                new_score["Total_char_pass_in_lowercase_in_prediction"] = total_upper_prediction

            if "D" or "P" or "L" or "U" or "X" in apply_transforms:
                new_score["Length_reference_transformed"] = len(self.reference_preprocess)
                new_score["Length_prediction_transformed"] = len(self.prediction_preprocess)


            # Add all scores to a final board
            self.scores.board = new_score

    def _compute_state_transformations(self, type_transform):
        """Compute scores for one transformation
        """
        transform = type_transform(
            [self.reference, self.prediction]
        )
        scores_transform = Scorer(
            transform[0],
            transform[1],
            truncate_score=self.truncate,
            show_percent=self.percent,
            round_digits=self.round_digits
        )

        return scores_transform

    def _compute_all_transformations(self, types_transforms):
        """Compute scores for all transformations
        """
        transform_for_all = _Composer(types_transforms)(
            [self.reference,
             self.prediction]
        )

        scores_transform_all = Scorer(
            transform_for_all[0],
            transform_for_all[1],
            truncate_score=self.truncate,
            show_percent=self.percent,
            round_digits=self.round_digits
        )

        return scores_transform_all, transform_for_all



import unittest

from kami.metrics.evaluation import Scorer

class testMetrics(unittest.TestCase):
    def setUp(self) -> None:
        self.reference = "Six semaines plus tard, Claude peignait un matin dans un flot de soleil qui tombait par la baie vitrée de l’atelier."
        self.prediction = "Six semaiNEs plus tard, lCCaude peignait un MA dans un flotille de soleil qui tombait baie vitrée de l’atelier."
        self.scorer_1 = Scorer(self.reference, self.reference, show_percent=True, truncate_score=True, round_digits='.001')
        self.scorer_2 = Scorer(self.reference, self.prediction, show_percent=True, truncate_score=True, round_digits='.001')
        self.scorer_3 = Scorer(self.reference, "", show_percent=True, truncate_score=True, round_digits='.001')

    def test_success_with_same_string(self):
         self.assertEqual(self.scorer_1.board, {'levensthein_distance_char': 0,
                                     'levensthein_distance_words': 0,
                                     'hamming_distance': 0,
                                     'wer': 0.0,
                                     'cer': 0.0,
                                     'wacc': 100.0,
                                     'wer_hunt': 0.0,
                                     'mer': 0.0,
                                     'cil': 0.0,
                                     'cip': 100.0,
                                     'hits': 116,
                                     'substitutions': 0,
                                     'deletions': 0,
                                     'insertions': 0,
                                     'Length_reference':len(self.reference),
                                     'Length_prediction':len(self.reference)})

    def test_success_with_change_string(self):
        self.assertEqual(self.scorer_2.board, {'levensthein_distance_char': 20,
                                          'levensthein_distance_words': 6,
                                          'hamming_distance': 'Ø',
                                          'wer': 28.571,
                                          'cer': 17.241,
                                          'wacc': 71.428,
                                          'wer_hunt': 23.809,
                                          'mer': 16.528,
                                          'cil': 20.775,
                                          'cip': 79.224,
                                          'hits': 101,
                                          'substitutions': 5,
                                          'deletions': 10,
                                          'insertions': 5,
                                     'Length_reference':len(self.reference),
                                     'Length_prediction':len(self.prediction)})


    def test_success_with_empty_string(self):
        self.assertEqual(self.scorer_3.board, {'levensthein_distance_char': 116,
                                          'levensthein_distance_words': 21,
                                          'hamming_distance': 'Ø',
                                          'wer': 100.0,
                                          'cer': 100.0, 
                                          'wacc': 0.0,
                                          'wer_hunt': 50.0,
                                          'mer': 100.0, 
                                          'cil': 0.0,
                                          'cip': 0,
                                          'hits': 0,
                                          'substitutions': 0,
                                          'deletions': 116,
                                          'insertions': 0,
                                     'Length_reference':len(self.reference),
                                     'Length_prediction':0})


    def test_noStringsToCompute(self):
        with self.assertRaises(TypeError):
            Scorer()





# -*- coding : utf-8 -*-

from os import path
import io
import setuptools
import subprocess

here = path.abspath(path.dirname(__file__))

kamilib_version = subprocess.run(['git', 'describe', '--tags'], stdout=subprocess.PIPE).stdout.decode("utf-8").strip()


try:
    with io.open(path.join(here, 'README.rst'), encoding='utf-8') as f:
        long_description = f.read()
except:
    long_description = u"HTR / OCR models evaluation agnostic Python package, originally based on the Kraken transcription system."


#with open("README.md", "r") as fh:
#    long_description = fh.read()

with open("requirements.txt", encoding="utf-8") as f:
    install_requires = f.read().splitlines()


CLASSIFIERS = [
        "Programming Language :: Python",
        "License :: OSI Approved :: MIT License",
        "Intended Audience :: Developers",
        "Intended Audience :: Education",
        "Intended Audience :: Science/Research",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules"
    ]


setuptools.setup(
    name="kamilib",
    version=kamilib_version,
    author="Lucas Terriel, Alix Chagué",
    author_email="lucas.terriel@inria.fr, alix.chague@inria.fr",
    license="MIT",
    description="HTR / OCR models evaluation agnostic Python package, originally based on the Kraken transcription system.",
    long_description="HTR / OCR models evaluation agnostic Python package, originally based on the Kraken transcription system.",
    url="https://github.com/KaMI-tools-project/KaMi-lib",
    packages=setuptools.find_packages(),
    install_requires=[
                        "termcolor==1.1.0",
                        "kraken==3.0.6",
                        "unidecode==1.3.4",
                        "protobuf==3.20.0",
                        "python-Levenshtein==0.12.2"
    ],
    python_requires='>=3.7',
    classifiers=CLASSIFIERS,
    keywords=["HTR", "OCR", "Evaluation framework", "metrics", "handwritten text recognition", "optical character recognition"]
)


# -*- coding: utf-8 -*-
# Authors : Alix Chagué <alix.chague@inria.fr>
#           Lucas Terriel <lucas.terriel@inria.fr>
# Licence : MIT
"""
    The ``XML Parser`` module parse XML PAGE / XML ALTO
    ===================================================

"""

from os.path import basename, isfile
from kraken.lib import xml, exceptions
from kraken.lib.xml import logger
from kami.kamutils._utils import (_report_log)

logger.disabled = True

__all__ = [
    "_XMLParser"
]

class _XMLParser:
    """A XML Parser for KaMI (ALTO/PAGE).

    Parameters
    ----------
        :param xml_path:  path to source file or plain text
        :type xml_path: str
        :param text_direction:  principal text direction for column ordering : "horizontal-lr", "horizontal-rl", "vertical-lr", "vertical-rl".
        :type text_direction: str
        :param script:  type of script.
        :type script: str

    Attributes
    ----------
        :ivar file_path: path to source XML file.
        :param file_path: str
        :ivar filename: source XML file name.
        :param text: str
        :ivar base_bounds: segmentation information extract from XML.
        :param base_bounds: dict
        :ivar TEXT_DIRECTION: principal text direction for column ordering : "horizontal-lr", "horizontal-rl", "vertical-lr", "vertical-rl".
        :param TEXT_DIRECTION: str
        :ivar script: type of script.
        :param script: str
        :ivar list_bounds: reformat segmentation information in `base_bounds`.
        :param list_bounds: list
        :ivar sentences: ground truth sentences in source XML.
        :param sentences: list
        :ivar content: ground truth text in sourece XML.
        :param content: str
    """

    def __init__(self, xml_path : str, text_direction : str, script : str) -> None:
        self.file_path = xml_path
        self.filename = basename(self.file_path) if isfile(self.file_path) else ""
        self.base_bounds = ""
        self.TEXT_DIRECTION = text_direction
        self.SCRIPT = script
        try:
            self.base_bounds = xml.parse_xml(self.file_path)
        except exceptions.KrakenInputException as eK:
            _report_log(f"Something went wrong while parsing XML content (XMLParser expects PAGE or ALTO XML content or a .xml file) : {eK}", "W")

        self.list_bounds = self._get_list_of_boundaries()
        self.sentences = self._get_content_textlines()
        self.content = "\n".join(self._get_content_textlines())
    
    def _get_content_textlines(self):
        """Extract sentences from bounds"""
        return [line['text'] for line in self.base_bounds['lines']]
    
    def _get_list_of_boundaries(self):
        """Reformat boundaries in list of dicts"""
        return [{
                'lines': [
                {
                    'baseline': bound['baseline'],
                    'boundary': bound['boundary'],
                    'text_direction': self.TEXT_DIRECTION,
                    'script': self.SCRIPT}
                    ],  
                'type': 'baselines',
            } for bound in self.base_bounds['lines']] 



# -*- coding : utf-8 -*-

from . import Kami

__name__ = "kami"
__title__ = "kamilib"
# __version__ = "0.1.2"
__licence__ = "MIT"
__doc__ = "HTR / OCR models evaluation agnostic Python package, originally based on the Kraken transcription system."
